"""Custom argparse type for LLMProvidersEnum validation."""

import argparse
from enum import StrEnum

from yt_summary.schemas.enums import LLMProvidersEnum, SummarisationModesEnum


def check_type(value: str, enum: type[StrEnum]) -> None:
    """Ensure the argument is a valid enum value.

    Args:
        value: The string to validate.
        enum: The enum type to validate against.

    Raises:
        argparse.ArgumentTypeError: If the value is not valid.

    """
    if value not in [e.value for e in enum]:
        raise argparse.ArgumentTypeError(f"Invalid value '{value}'. Must be one of: {[e.value for e in enum]}")


def check_provider_type(provider: str) -> None:
    """Ensure the provider argument is a valid LLMProvidersEnum value.

    Args:
        provider (str): The provider string to validate.

    Raises:
        argparse.ArgumentTypeError: If the provider is not valid.

    """
    return check_type(provider, LLMProvidersEnum)


def check_mode_type(mode: str) -> None:
    """Ensure the mode argument is a valid SummarisationModesEnum value.

    Args:
        mode (str): The mode string to validate.

    Raises:
        argparse.ArgumentTypeError: If the mode is not valid.

    """
    return check_type(mode, SummarisationModesEnum)



"""Runner functions to simplify usage of the package."""

from requests import Session
from youtube_transcript_api.proxies import ProxyConfig

from yt_summary.extractors.transcript import TranscriptExtractor
from yt_summary.schemas.enums import LLMProvidersEnum, SummarisationModesEnum
from yt_summary.schemas.models import LLMModel
from yt_summary.summarisers.refined_summariser import RefinedSummariser
from yt_summary.summarisers.simple_summariser import SimpleSummariser


async def get_youtube_summary(
    url: str,
    llm_provider: LLMProvidersEnum = LLMProvidersEnum.OPENAI,
    model_name: str = "gpt-5-mini-2025-08-07",
    mode: SummarisationModesEnum = SummarisationModesEnum.COMPACT,
    languages: list[str] | None = None,
    *,
    preserve_formatting: bool = False,
    proxy_config: ProxyConfig | None = None,
    http_client: Session | None = None,
) -> str:
    """Run the summarisation pipeline.

    Args:
        url: The YouTube video URL.
        llm_provider: The LLM provider to use. Defaults to "openai".
        model_name: The model name to use. Defaults to "gpt-5-mini-2025-08-07".
        mode: The summarisation mode. Defaults to "simple".
        languages: A list of language codes in a descending priority. For
            example, if this is set to ["de", "en"] it will first try to fetch the
            german transcript (de) and then fetch the english transcript (en) if
            it fails to do so. This defaults to ["en"].
        preserve_formatting: whether to keep select HTML text formatting
        proxy_config: Optional ProxyConfig for youtube-transcript-api
        http_client: Optional requests.Session for youtube-transcript-api

    Returns:
        str: The summary of the YouTube video.

    """
    transcript = await TranscriptExtractor(proxy_config=proxy_config, http_client=http_client).fetch(
        url, languages=languages if languages else ["en"], preserve_formatting=preserve_formatting
    )
    match mode:
        case SummarisationModesEnum.COMPACT:
            summariser = SimpleSummariser(llm=LLMModel(provider=LLMProvidersEnum(llm_provider), model=model_name))
        case SummarisationModesEnum.REFINED:
            summariser = RefinedSummariser(llm=LLMModel(provider=LLMProvidersEnum(llm_provider), model=model_name))
    return await summariser.summarise(transcript)



"""High level summary."""

from llama_index.core import Document, DocumentSummaryIndex, PromptTemplate, get_response_synthesizer
from llama_index.core.node_parser import SentenceSplitter
from llama_index.core.response_synthesizers.type import ResponseMode

from yt_summary.schemas.models import YoutubeTranscriptRaw
from yt_summary.summarisers.base_summariser import BaseSummariser
from yt_summary.summarisers.templates import COMPACT_SUMMARY_QA_PROMPT_TEMPLATE


class SimpleSummariser(BaseSummariser):
    """Class for generating high level summaries from video transcripts."""

    async def summarise(self, transcript: YoutubeTranscriptRaw) -> str:
        """Generate a high level summary from the provided text.

        Args:
            transcript: The transcript to summarise.

        Returns:
            A high level summary of the transcript.

        """
        docs = Document(
            text=transcript.text, metadata=transcript.metadata.model_dump(), doc_id=transcript.metadata.video_id
        )
        splitter = SentenceSplitter(chunk_size=2048, chunk_overlap=200)
        response_synthesizer = get_response_synthesizer(
            llm=self.model.llm,
            text_qa_template=PromptTemplate(COMPACT_SUMMARY_QA_PROMPT_TEMPLATE),
            response_mode=ResponseMode.COMPACT,
            use_async=True,
        )
        index = DocumentSummaryIndex.from_documents(
            [docs],
            llm=self.model.llm,
            transformations=[splitter],
            response_synthesizer=response_synthesizer,
            embed_model=self.model.embed_model,
        )
        return index.get_document_summary(transcript.metadata.video_id)



"""Exceptions raised by yt-summary."""


class YTSummaryExpection(Exception):
    """Base class for all exceptions raised by yt-summary."""


class MetadataNotFoundException(YTSummaryExpection):
    """Raised when no metadata is found for the provided URL or video ID."""

    def __init__(self, aiohttp_client_error: Exception) -> None:
        message = "Failed to fetch the metadata. Http error: \n"
        super().__init__(f"{message}{aiohttp_client_error}")



"""Command line interface for the YouTube summary tool."""

import argparse
import os
import sys
from importlib.metadata import PackageNotFoundError, version


class YTSummaryCLI:
    """Command line interface for the YouTube summary tool."""

    def __init__(self, args: list[str]) -> None:
        self._args = args

    async def run(self) -> str:
        """Run the CLI tool."""
        parsed_args = self._parse_args()

        if parsed_args.list_providers:
            from yt_summary.schemas.enums import LLMProvidersEnum

            return f"Supported LLM Providers: {[e.value for e in LLMProvidersEnum]}"

        if not parsed_args.url:
            return "Please provide a YouTube video URL or ID."

        try:
            if parsed_args.mode:
                from yt_summary.cli.errors import check_mode_type

                check_mode_type(parsed_args.mode)

            if parsed_args.provider:
                from yt_summary.cli.errors import check_provider_type

                check_provider_type(parsed_args.provider)

            from yt_summary.llm_config import llm_configs

            llm_config = llm_configs[parsed_args.provider]

            if not os.getenv(llm_config.key_name):
                return f"{llm_config.key_name} environment variable not set."

            from yt_summary.extractors.transcript import TranscriptExtractor
            from yt_summary.run.getters import summarisers
            from yt_summary.schemas.models import LLMModel

            llm_model = LLMModel(
                provider=parsed_args.provider,
                model=parsed_args.model or llm_config.default_model,
            )
            transcript_extractor = TranscriptExtractor()
            transcript = await transcript_extractor.fetch(parsed_args.url)
            summariser = summarisers[parsed_args.mode](llm=llm_model)
            return f"\n {await summariser.summarise(transcript)}"

        except Exception as e:
            return str(e)

    def _get_version(self) -> str:
        try:
            return version("yt-pls")
        except PackageNotFoundError:
            return "unknown"

    def _parse_args(self) -> argparse.Namespace:
        parser = argparse.ArgumentParser(prog="yt-summary", description="Youtube Summariser")
        parser.add_argument(
            "url",
            nargs="?",
            type=str,
            help=("video ID or URL of the Youtube video to summarize"),
        )

        parser.add_argument(
            "--version",
            "-v",
            action="version",
            version=f"%(prog)s {self._get_version()}",
            help="show the version number and exit",
        )

        parser.add_argument(
            "--list-providers",
            action="store_true",
            help="list all supported language model providers and exit",
        )

        parser.add_argument(
            "--provider",
            "-p",
            type=str,
            default="openai",
            help="language model provider to use. (default: openai)",
        )

        parser.add_argument(
            "--model",
            type=str,
            default=None,
            help="model name for the provider",
        )

        parser.add_argument(
            "--mode",
            "-m",
            type=str,
            default="compact",
            help=(
                "summarization mode: `compact` or `refined` (default: compact). "
                "`compact`: List metadata, high level summary and Q&A with timestamps. "
                "It utilises the `DocumentSummaryIndex` from LLamaIndex. "
                "`refined`: List metadata, high level summary and a detailed, timestamped summary of key points. "
                "It chunks the transcript (chunk_size=4096) and generates summaries for each chunk before"
                "consolidating them by making multiple calls to the LLM asynchronously. "
                "Be aware of the rate limits of your chosen LLM provider."
            ),
        )

        if len(sys.argv) == 1:
            parser.print_help()
            sys.exit(1)

        return parser.parse_args(self._args)



"""Extract metadata from YouTube videos using yt-dlp."""

import aiohttp
from pydantic import HttpUrl

from yt_summary.schemas.exceptions import MetadataNotFoundException
from yt_summary.schemas.models import YoutubeMetadata
from yt_summary.utils.misc import parse_youtube_video_id


async def extract_metadata(url: str) -> YoutubeMetadata:
    """Extract metadata from a YouTube video URL.

    Args:
        url: The YouTube video URL.

    Returns:
        A tuple containing the YoutubeMetadata object and subtitles dictionary (if available).

    Raises:
        MetadataNotFoundError: If there is an error fetching the metadata.

    """
    if "youtube.com" not in url and "youtu.be" not in url:
        url = f"https://www.youtube.com/watch?v={url}"

    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(
                f"https://www.youtube.com/oembed?url={url}&format=json",
                headers={"Accept": "application/json"},
                raise_for_status=True,
            ) as response:
                metadata = await response.json()
    except aiohttp.ClientError as e:
        raise MetadataNotFoundException(e) from e

    return YoutubeMetadata(
        video_id=parse_youtube_video_id(url),
        title=metadata["title"],
        author=metadata["author_name"],
        channel_id=metadata["author_url"].split("@")[-1],
        video_url=HttpUrl(url.split("&")[0]),
        channel_url=HttpUrl(metadata["author_url"]),
        thumbnail_url=HttpUrl(metadata.get("thumbnail_url")),
    )



"""Prompt templates for video summarization tasks."""

REFINED_SUMMARY_CHUCKED_PROMPT = """
    You are given a portion of a youtube transcript with timestamps.
    The timestamps are placed in the begining of sentences in the
    format [%H:%M:%S or %M:%S (timestamp in seconds)].

    Create a very detailed and comphrensive summary of the topics in this section with their corresponding timestamps.
    Keep the examples and analogies if any.

    FORMATTING RULES:
    - Extract timestamps from [timestamp] markers
    - Format each point as: [timestamp] Summary text
    - Focus on main topics, arguments, and examples
    - Be comprehensive
    - Only include the start timestamp for each key point.
    - Ignore sponsor blocks

    Transcript section:
        {chunk}

    Here is the timestamped summary of key points from this section:
"""


REFINED_CONSOLIDATION_PROMPT = """
    You are given the metadata and multiple timestamped summaries from different sections
    of a video transcript which are overlaped.
    Organize and consolidate these into a coherent, comprehensive summary.

    RULES:
    1. Format the output as marked down with headers and bullet points.
    2. Give the Youtube metada at the top as a **list**, including: title, author, channel id, url, channel url,
        thumbnail url, language.
    3. Give an overall summary of the transcript.
    4. Breakdown into sections and subsections with bullet points for topics. Each bullet point
        should be a summary of a broad topic instead of an one-liner.
    5. Omit adjacent timestamps that refer to the same topic.
    6. Remove any redundancy
    7. Maintain chronological order
    8. Provide a Q&A section at the end with insightful questions and their answers based on the transcript.
    9. Add line breaks immediately after headers and subheaders.

    OUTPUT STRUCTURE:

    # Author: Video Title
    ## Metadata
        - title
        - author
        - channel id
        - url
        - channel url
        - thumbnail url
        - language

    ## Summary

    ## Sections
    ### Subsection 1
        - [timestamp 1] topic 1 for Header 1
        - [timestamp 2] topic 2 for Header 1
        - etc.
    ### Subsection 2
        - [timestamp 3] topic 1 for Header 2
        - etc.
    - etc.

    ## Q&A
    ### Question 1
        Your Ansewer
    ### Question 2
        Your Ansewer
    etc.

    Section summaries:
        {combined_summary}

    Here is the metadaata, high level summary as well as the timestamped summary of key points:

    ... Your Reponse ...

    END CONVERSATION.
    """


COMPACT_SUMMARY_QA_PROMPT_TEMPLATE = """
    You are an expert summarizer and question answerer.
    Given the following document, give the metadata of the video such as title and author and
    generate a concise, high-level summary.
    After the summary, generate a list of insightful questions that this document can answer,
    and provide detailed and accurate answers to each.
    There's no need prompt the user to ask questions, as this is a self-contained summary and Q&A.

    RULES:
    1. Format the output as marked down with headers and bullet points.
    2. Give the Youtube metada at the top as a **list**, including: title, author, channel id, url, channel url,
        thumbnail url, language.
    3. Give an overall summary of the transcript.
    4. Breakdown into sections and subsections with bullet points for topics. Each bullet point
        should be a summary of a broad topic instead of an one-liner.
    5. Omit adjacent timestamps that refer to the same broad topic.
    6. Remove any redundancy
    7. Maintain chronological order
    8. Provide a Q&A section at the end with insightful questions and their answers based on the transcript.
    9. Add line breaks immediately after headers and subheaders.

    OUTPUT STRUCTURE:

        # Author: Video Title

        ## Metadata

            - title
            - author
            - channel id
            - url
            - channel url
            - thumbnail url
            - language

        ## Summary

        High level summary. If the transcript is long, make the summary more comprehensive.


        ## Sections

        ### Subsection 1

            - [timestamp 1] Broad topic 1 for Header 1
            - [timestamp 2] Broad topic 2 for Header 1
            - etc.
        ### Subsection 2

            - [timestamp 3] Broad topic 1 for Header 2
            - etc.
        - etc.

        ## Q&A

        ### e.g. What is ...?

            Your Ansewer
        ### e.g. Why does ...?

            Your Ansewer
        etc.


    Document:
        {context_str}

    Here is the metadaata, high level summary as well as the timestamped summary of key points:

    ... Your Reponse ...

    END CONVERSATION.
"""



"""Enums."""

from enum import StrEnum


class LLMProvidersEnum(StrEnum):
    """Enum for different Large Language Models (LLMs)."""

    OPENAI = "openai"
    GOOGLE = "google"
    ANTHROPIC = "anthropic"


class SummarisationModesEnum(StrEnum):
    """Enum for different summarization modes.

    Attributes:
        COMPACT: Simple summarisation using the `COMPACT` response mode in LlamaIndex.
        REFINE: Refined summarisation by iteratively refining the summary with each chunk of text
        that fed to the LLM directly.

    """

    COMPACT = "compact"
    REFINED = "refined"



"""Enviroment configuration for the application."""

from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    """Settings."""

    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        extra="ignore",
    )

    # FASTAPI (future use)
    API_TITLE: str = "Youtube Rag"
    API_DESCRIPTION: str = "A simple Youtube RAG application"
    API_VERSION: str = "1.0.0"
    BACKEND_CORS_ORIGINS: list[str] = ["http://localhost"]
    DOCS_URL: str = "/docs"
    REDOC_URL: str = "/redoc"

    # LLM API
    OPENAI_API_KEY: str = ""
    OPENAI_MODEL: str = "gpt-5-mini-2025-08-07"
    GOOGLE_API_KEY: str = ""
    GOOGLE_MODEL: str = "gemini-1.5-flash"
    ANTHROPIC_API_KEY: str = ""
    ANTHROPIC_MODEL: str = "claude-3-5-haiku-20241022"

    MIGRATIONS_FOLDER_PATH: str = "migrations"


settings = Settings()



"""Pydantic models."""

from llama_index.core.llms.function_calling import FunctionCallingLLM
from llama_index.embeddings.openai.base import BaseEmbedding
from pydantic import BaseModel, HttpUrl

from yt_summary.schemas.enums import LLMProvidersEnum


class YoutubeMetadata(BaseModel):
    """YouTube video metadata model.

    Attributes:
        video_id: The unique identifier for the YouTube video.
        title: The title of the YouTube video.
        author: The author of the YouTube video.
        channel_id: The unique identifier for the YouTube channel.
        video_url: The URL of the YouTube video.
        channel_url: The URL of the YouTube channel.
        thumbnail_url: The URL of the YouTube video's thumbnail image.

    """

    video_id: str
    title: str
    author: str
    channel_id: str
    video_url: HttpUrl
    channel_url: HttpUrl
    thumbnail_url: HttpUrl | None = None
    is_generated: bool | None = None
    language: str | None = None
    language_code: str | None = None


class YoutubeTranscriptRaw(BaseModel):
    """YouTube video metadata with raw transcript.

    Attributes:
        metadata: The metadata of the YouTube video.
        text: transcript text.

    """

    metadata: YoutubeMetadata
    text: str


class LLMModel(BaseModel):
    """Language model configuration.

    Attributes:
        provider: The provider of the language model.
        model: The specific model to use.

    """

    provider: LLMProvidersEnum
    model: str


class LLMAndEmbeddingModel(BaseModel):
    """Language model and embedding model pair.

    Attributes:
        llm: The language model instance.
        embed_model: The embedding model instance.

    """

    llm: FunctionCallingLLM
    embed_model: BaseEmbedding



"""Entry point for the CLI."""

import asyncio
import sys

from yt_summary.cli.cli import YTSummaryCLI


def main() -> None:
    """Run the CLI tool."""
    print(asyncio.run(YTSummaryCLI(sys.argv[1:]).run()))


if __name__ == "__main__":
    main()



"""Configuration for different language models."""

from pydantic import BaseModel

from yt_summary.config import settings
from yt_summary.schemas.enums import LLMProvidersEnum


class LLMConfigs(BaseModel):
    """Language model configurations.

    Attributes:
        key_env: Environment variable name for the API key.
        default_key: Default API key if environment variable is not set.
        default_model: Default model name to use.

    """

    key_name: str
    default_key: str
    default_model: str


class OPENAIConfig(LLMConfigs):
    """OpenAI model configuration."""

    key_name: str = "OPENAI_API_KEY"
    default_key: str = settings.OPENAI_API_KEY
    default_model: str = settings.OPENAI_MODEL


class GOOGLEConfig(LLMConfigs):
    """Google model configuration."""

    key_name: str = "GOOGLE_API_KEY"
    default_key: str = settings.GOOGLE_API_KEY
    default_model: str = settings.GOOGLE_MODEL


class ANTHROPICConfig(LLMConfigs):
    """Anthropic model configuration."""

    key_name: str = "ANTHROPIC_API_KEY"
    default_key: str = settings.ANTHROPIC_API_KEY
    default_model: str = settings.ANTHROPIC_MODEL


llm_configs: dict[LLMProvidersEnum, LLMConfigs] = {
    LLMProvidersEnum.OPENAI: OPENAIConfig(),
    LLMProvidersEnum.GOOGLE: GOOGLEConfig(),
    LLMProvidersEnum.ANTHROPIC: ANTHROPICConfig(),
}



"""Async helper functions."""

import asyncio
from concurrent.futures import ThreadPoolExecutor
from functools import partial
from typing import Any, Callable

executor = ThreadPoolExecutor()


async def to_async(func: Callable[..., Any], *args: Any, **kwargs: dict[str, Any]) -> Any:  # noqa: ANN401
    """Run a blocking function in an async context using ThreadPoolExecutor.

    Args:
        func: The blocking function to run.
        *args: Positional arguments to pass to the function.
        **kwargs: Keyword arguments to pass to the function.

    """
    loop = asyncio.get_event_loop()
    func_with_args = partial(func, *args, **kwargs)
    return await loop.run_in_executor(executor, func_with_args)



"""Timestamped summariser module."""

import asyncio

from llama_index.core.node_parser import SentenceSplitter

from yt_summary.schemas.models import YoutubeTranscriptRaw
from yt_summary.summarisers.base_summariser import BaseSummariser
from yt_summary.summarisers.templates import REFINED_CONSOLIDATION_PROMPT, REFINED_SUMMARY_CHUCKED_PROMPT


class RefinedSummariser(BaseSummariser):
    """Class for generating refined timestamped summaries from video transcripts."""

    async def summarise(self, transcript: YoutubeTranscriptRaw) -> str:
        """Generate a timestamped summary from the provided text.

        Args:
            transcript: The transcript to summarise.

        Returns:
            A timestamped summary of the transcript.

        """
        chunks = SentenceSplitter(chunk_size=4096, chunk_overlap=200).split_text(transcript.text)
        tasks = [self.model.llm.acomplete(REFINED_SUMMARY_CHUCKED_PROMPT.format(chunk=chunk)) for chunk in chunks]
        section_texts = (
            f"{transcript.metadata.model_dump_json()} \n"
            f"{'\n'.join([s.text.strip() for s in await asyncio.gather(*tasks)])}"
        )
        final_response = await self.model.llm.acomplete(
            REFINED_CONSOLIDATION_PROMPT.format(combined_summary=section_texts)
        )
        return final_response.text



"""Base class for text summarisation models."""

import abc
import os

from yt_summary.llm_config import llm_configs
from yt_summary.schemas.enums import LLMProvidersEnum
from yt_summary.schemas.models import LLMAndEmbeddingModel, LLMModel, YoutubeTranscriptRaw


class BaseSummariser(abc.ABC):
    """Base class for text summarisation models.

    Attributes:
        model: LLM model to use for summarisation.

    """

    def __init__(self, llm: LLMModel) -> None:
        self.model = self._get_model(llm)

    @staticmethod
    def _get_model(llm: LLMModel) -> LLMAndEmbeddingModel:
        """Get the LLM model based on the provided configuration.

        Args:
            llm: LLM model model.

        Returns:
            An instance of the selected LLM model.

        """
        opts = llm_configs[llm.provider]
        os.environ[opts.key_name] = os.getenv(opts.key_name) or opts.default_key
        embed_model = None
        match llm.provider:
            case LLMProvidersEnum.OPENAI:
                from llama_index.embeddings.openai import OpenAIEmbedding
                from llama_index.llms.openai import OpenAI

                llm_model = OpenAI(temperature=0, model=llm.model)
                embed_model = OpenAIEmbedding()
            case LLMProvidersEnum.GOOGLE:
                from llama_index.embeddings.google_genai import GoogleGenAIEmbedding
                from llama_index.llms.google_genai import GoogleGenAI

                llm_model = GoogleGenAI(temperature=0, model=llm.model)
                embed_model = GoogleGenAIEmbedding()
            case LLMProvidersEnum.ANTHROPIC:
                from llama_index.embeddings.openai import OpenAIEmbedding
                from llama_index.llms.anthropic import Anthropic

                llm_model = Anthropic(temperature=0, model=llm.model)
                embed_model = OpenAIEmbedding()
        return LLMAndEmbeddingModel(llm=llm_model, embed_model=embed_model)

    @abc.abstractmethod
    async def summarise(self, transcript: YoutubeTranscriptRaw) -> str:
        """Abstract method to summarise text.

        Args:
            transcript: The transcript text to summarise.

        Returns:
            Summarised text.

        """



"""Getters."""

from yt_summary.schemas.enums import SummarisationModesEnum
from yt_summary.summarisers.base_summariser import BaseSummariser
from yt_summary.summarisers.refined_summariser import RefinedSummariser
from yt_summary.summarisers.simple_summariser import SimpleSummariser

summarisers: dict[SummarisationModesEnum, type[BaseSummariser]] = {
    SummarisationModesEnum.COMPACT: SimpleSummariser,
    SummarisationModesEnum.REFINED: RefinedSummariser,
}



"""Miscellaneous utility functions."""

import time


def convert_to_readable_time(seconds: int) -> str:
    """Convert seconds to a human-readable time format (hh:mm:ss or mm:ss).

    Args:
        seconds (int): The number of seconds to convert.

    Returns:
        str: The time in hh:mm:ss format if >= 1 hour, otherwise mm:ss format.

    """
    if seconds >= 3600:
        return time.strftime("%H:%M:%S", time.gmtime(seconds))
    return time.strftime("%M:%S", time.gmtime(seconds))


def parse_youtube_video_id(url: str) -> str:
    """Attempt to extract the Youtube video ID from a URL or return the ID if given directly.

    Args:
        url: The Youtube video URL or ID.

    Returns:
        The extracted video ID.

    """
    if "youtube.com" in url:
        return url.split("?v=")[-1].split("&")[0]

    if "youtu.be" in url:
        return url.split("/")[-1].split("?")[0]
    return url



"""Fetch transcript."""

import asyncio
from typing import Iterable

from requests import Session
from youtube_transcript_api import FetchedTranscript, YouTubeTranscriptApi
from youtube_transcript_api.proxies import ProxyConfig

from yt_summary.extractors.metadata import extract_metadata
from yt_summary.schemas.models import YoutubeTranscriptRaw
from yt_summary.utils.async_helpers import to_async
from yt_summary.utils.misc import convert_to_readable_time, parse_youtube_video_id


class TranscriptExtractor:
    """Transcript extractor. A wrapper around YouTubeTranscriptApi.

    Attributes:
        ytt_api: YouTubeTranscriptApi instance

    """

    def __init__(self, proxy_config: ProxyConfig | None = None, http_client: Session | None = None) -> None:
        self.ytt_api = YouTubeTranscriptApi(proxy_config=proxy_config, http_client=http_client)

    async def fetch(
        self, url: str, languages: Iterable[str] | None = None, *, preserve_formatting: bool = False
    ) -> YoutubeTranscriptRaw:
        """Asynchronously fetch transcript.

        Args:
            url: video URL or ID you want to retrieve the transcript for.
            languages: A list of language codes in a descending priority. For
                example, if this is set to ["de", "en"] it will first try to fetch the
                german transcript (de) and then fetch the english transcript (en) if
                it fails to do so. This defaults to ["en"].
            sentences_per_timestamp_group: number of sentences to stitch together. To
                improve granularity, reduce this number.
            preserve_formatting: whether to keep select HTML text formatting

        Returns:
            The transcript text.

        """
        fetched_transcript, metadata = await asyncio.gather(
            self._afetch_transcript(
                url,
                languages=languages,
                preserve_formatting=preserve_formatting,
            ),
            extract_metadata(url),
        )
        text = " ".join([f"[{convert_to_readable_time(int(s.start))}] {s.text}" for s in fetched_transcript.snippets])
        metadata.is_generated = fetched_transcript.is_generated
        metadata.language = fetched_transcript.language
        metadata.language_code = fetched_transcript.language_code
        return YoutubeTranscriptRaw(
            metadata=metadata,
            text=text,
        )

    def fetch_transcript(
        self, url: str, languages: Iterable[str] | None = None, *, preserve_formatting: bool = False
    ) -> FetchedTranscript:
        """Fetch transcript.

        Args:
            url: video URL or ID you want to retrieve the transcript for.
            languages: A list of language codes in a descending priority. For
                example, if this is set to ["de", "en"] it will first try to fetch the
                german transcript (de) and then fetch the english transcript (en) if
                it fails to do so. This defaults to ["en"].
            preserve_formatting: whether to keep select HTML text formatting

        Returns:
            The transcript text.

        """
        video_id = parse_youtube_video_id(url)
        return self.ytt_api.fetch(
            video_id,
            languages=languages if languages else ["en"],
            preserve_formatting=preserve_formatting,
        )

    async def _afetch_transcript(
        self, url: str, languages: Iterable[str] | None = None, *, preserve_formatting: bool = False
    ) -> FetchedTranscript:
        """Asynchronously fetch transcript.

        Args:
            url: video URL or ID you want to retrieve the transcript for.
            languages: A list of language codes in a descending priority. For
                example, if this is set to ["de", "en"] it will first try to fetch the
                german transcript (de) and then fetch the english transcript (en) if
                it fails to do so. This defaults to ["en"].
            preserve_formatting: whether to keep select HTML text formatting

        Returns:
            The transcript text.

        """
        kwargs = {
            "url": url,
            "languages": languages,
            "preserve_formatting": preserve_formatting,
        }
        return await to_async(self.fetch_transcript, **kwargs)



"""Response models for summarisers."""

from pydantic import BaseModel


class TimestampResponse(BaseModel):
    """Response model for timestamped summary.

    Attributes:
        timestamp: Timestamp in seconds.
        text: Summary text.

    """

    timestamp: int
    text: str


class TimestampSummary(BaseModel):
    """Response model for timestamped summary.

    Attributes:
        summary: Overall summary of the transcript.
        responses: List of timestamped summary points.

    """

    summary: str
    repsonses: list[TimestampResponse]
